@@ -17,21 +17,6 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
-/**
- * SECTION:element-appsrc
- *
- * The appsrc element can be used by applications to insert data into a
- * GStreamer pipeline. Unlike most GStreamer elements, Appsrc provides
- * external API functions.
- *
- * For the documentation of the API, please see the
- * <link linkend="gst-plugins-base-libs-appsrc">libgstapp</link> section in the
- * GStreamer Plugins Base Libraries documentation.
- * 
- * Since: 0.10.22
- */
-
 /**
  * SECTION:gstappsrc
  * @short_description: Easy way for applications to inject buffers into a
@@ -52,7 +37,7 @@
  * byte buffers.
  *
  * The main way of handing data to the appsrc element is by calling the
- * gst_app_src_push_buffer() method or by emiting the push-buffer action signal.
+ * gst_app_src_push_buffer() method or by emitting the push-buffer action signal.
  * This will put the buffer onto a queue from which appsrc will read from in its
  * streaming thread. It is important to note that data transport will not happen
  * from the thread that performed the push-buffer call.
@@ -64,7 +49,7 @@
  * block the push-buffer method until free data becomes available again.
  *
  * When the internal queue is running out of data, the "need-data" signal is
- * emited, which signals the application that it should start pushing more data
+ * emitted, which signals the application that it should start pushing more data
  * into appsrc.
  *
  * In addition to the "need-data" and "enough-data" signals, appsrc can emit the
@@ -77,7 +62,7 @@
  * These signals allow the application to operate the appsrc in two different
  * ways:
  *
- * The push model, in which the application repeadedly calls the push-buffer method
+ * The push model, in which the application repeatedly calls the push-buffer method
  * with a new buffer. Optionally, the queue size in the appsrc can be controlled
  * with the enough-data and need-data signals by respectively stopping/starting
  * the push-buffer calls. This is a typical mode of operation for the
@@ -95,7 +80,7 @@
  * For the stream and seekable modes, setting this property is optional but
  * recommended.
  *
- * When the application is finished pushing data into appsrc, it should call 
+ * When the application is finished pushing data into appsrc, it should call
  * gst_app_src_end_of_stream() or emit the end-of-stream action signal. After
  * this call, no more buffers can be pushed into appsrc until a flushing seek
  * happened or the state of the appsrc has gone through READY.
@@ -117,6 +102,8 @@
 #include "gstapp-marshal.h"
 #include "gstappsrc.h"
 
+#include "gst/glib-compat-private.h"
+
 struct _GstAppSrcPrivate
 {
   GCond *cond;
@@ -198,23 +185,24 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS_ANY);
 
-
-#define GST_TYPE_APP_STREAM_TYPE (stream_type_get_type ())
-static GType
-stream_type_get_type (void)
+GType
+gst_app_stream_type_get_type (void)
 {
-  static GType stream_type_type = 0;
+  static volatile gsize stream_type_type = 0;
   static const GEnumValue stream_type[] = {
-    {GST_APP_STREAM_TYPE_STREAM, "Stream", "stream"},
-    {GST_APP_STREAM_TYPE_SEEKABLE, "Seekable", "seekable"},
-    {GST_APP_STREAM_TYPE_RANDOM_ACCESS, "Random Access", "random-access"},
-    {0, NULL, NULL},
+    {GST_APP_STREAM_TYPE_STREAM, "GST_APP_STREAM_TYPE_STREAM", "stream"},
+    {GST_APP_STREAM_TYPE_SEEKABLE, "GST_APP_STREAM_TYPE_SEEKABLE", "seekable"},
+    {GST_APP_STREAM_TYPE_RANDOM_ACCESS, "GST_APP_STREAM_TYPE_RANDOM_ACCESS",
+        "random-access"},
+    {0, NULL, NULL}
   };
 
-  if (!stream_type_type) {
-    stream_type_type = g_enum_register_static ("GstAppStreamType", stream_type);
+  if (g_once_init_enter (&stream_type_type)) {
+    GType tmp = g_enum_register_static ("GstAppStreamType", stream_type);
+    g_once_init_leave (&stream_type_type, tmp);
   }
-  return stream_type_type;
+
+  return (GType) stream_type_type;
 }
 
 static void gst_app_src_uri_handler_init (gpointer g_iface,
@@ -271,11 +259,11 @@ gst_app_src_base_init (gpointer g_class)
   GST_DEBUG_CATEGORY_INIT (app_src_debug, "appsrc", 0, "appsrc element");
 
   gst_element_class_set_details_simple (element_class, "AppSrc",
-      "Generic/Src", "Allow the application to feed buffers to a pipeline",
+      "Generic/Source", "Allow the application to feed buffers to a pipeline",
       "David Schleef <ds@schleef.org>, Wim Taymans <wim.taymans@gmail.com>");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_app_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_app_src_template);
 }
 
 static void
@@ -347,7 +335,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
   /**
    * GstAppSrc::block
    *
-   * When max-bytes are queued and after the enough-data signal has been emited,
+   * When max-bytes are queued and after the enough-data signal has been emitted,
    * block any further push-buffer calls until the amount of queued bytes drops
    * below the max-bytes limit.
    */
@@ -420,7 +408,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
 
   /**
    * GstAppSrc::need-data:
-   * @appsrc: the appsrc element that emited the signal
+   * @appsrc: the appsrc element that emitted the signal
    * @length: the amount of bytes needed.
    *
    * Signal that the source needs more data. In the callback or from another
@@ -439,11 +427,11 @@ gst_app_src_class_init (GstAppSrcClass * klass)
 
   /**
    * GstAppSrc::enough-data:
-   * @appsrc: the appsrc element that emited the signal
+   * @appsrc: the appsrc element that emitted the signal
    *
    * Signal that the source has enough data. It is recommended that the
    * application stops calling push-buffer until the need-data signal is
-   * emited again to avoid excessive buffer queueing.
+   * emitted again to avoid excessive buffer queueing.
    */
   gst_app_src_signals[SIGNAL_ENOUGH_DATA] =
       g_signal_new ("enough-data", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
@@ -452,7 +440,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
 
   /**
    * GstAppSrc::seek-data:
-   * @appsrc: the appsrc element that emited the signal
+   * @appsrc: the appsrc element that emitted the signal
    * @offset: the offset to seek to
    *
    * Seek to the given offset. The next push-buffer should produce buffers from
@@ -489,7 +477,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
     * GstAppSrc::end-of-stream:
     * @appsrc: the appsrc
     *
-    * Notify @appsrc that no more buffer are available. 
+    * Notify @appsrc that no more buffer are available.
     */
   gst_app_src_signals[SIGNAL_END_OF_STREAM] =
       g_signal_new ("end-of-stream", G_TYPE_FROM_CLASS (klass),
@@ -996,8 +984,10 @@ gst_app_src_create (GstBaseSrc * bsrc, guint64 offset, guint size,
       /* only update the offset when in random_access mode */
       if (priv->stream_type == GST_APP_STREAM_TYPE_RANDOM_ACCESS)
         priv->offset += buf_size;
-      *buf = gst_buffer_make_metadata_writable (*buf);
-      gst_buffer_set_caps (*buf, caps);
+      if (caps) {
+        *buf = gst_buffer_make_metadata_writable (*buf);
+        gst_buffer_set_caps (*buf, caps);
+      }
 
       /* signal that we removed an item */
       g_cond_broadcast (priv->cond);
@@ -1022,7 +1012,7 @@ gst_app_src_create (GstBaseSrc * bsrc, guint64 offset, guint size,
        * random-access mode (where a buffer is normally pushed in the above
        * signal) we can still be empty because the pushed buffer got flushed or
        * when the application pushes the requested buffer later, we support both
-       * possiblities. */
+       * possibilities. */
       if (!g_queue_is_empty (priv->queue))
         continue;
 
@@ -1080,7 +1070,7 @@ seek_error:
  * a copy of the caps structure. After calling this method, the source will
  * only produce caps that match @caps. @caps must be fixed and the caps on the
  * buffers must match the caps or left NULL.
- * 
+ *
  * Since: 0.10.22
  */
 void
@@ -1113,7 +1103,7 @@ gst_app_src_set_caps (GstAppSrc * appsrc, const GstCaps * caps)
  * Get the configured caps on @appsrc.
  *
  * Returns: the #GstCaps produced by the source. gst_caps_unref() after usage.
- * 
+ *
  * Since: 0.10.22
  */
 GstCaps *
@@ -1141,8 +1131,8 @@ gst_app_src_get_caps (GstAppSrc * appsrc)
  * @size: the size to set
  *
  * Set the size of the stream in bytes. A value of -1 means that the size is
- * not known. 
- * 
+ * not known.
+ *
  * Since: 0.10.22
  */
 void
@@ -1165,10 +1155,10 @@ gst_app_src_set_size (GstAppSrc * appsrc, gint64 size)
  * @appsrc: a #GstAppSrc
  *
  * Get the size of the stream in bytes. A value of -1 means that the size is
- * not known. 
+ * not known.
  *
  * Returns: the size of the stream previously set with gst_app_src_set_size();
- * 
+ *
  * Since: 0.10.22
  */
 gint64
@@ -1197,8 +1187,8 @@ gst_app_src_get_size (GstAppSrc * appsrc)
  * Set the stream type on @appsrc. For seekable streams, the "seek" signal must
  * be connected to.
  *
- * A stream_type stream 
- * 
+ * A stream_type stream
+ *
  * Since: 0.10.22
  */
 void
@@ -1224,7 +1214,7 @@ gst_app_src_set_stream_type (GstAppSrc * appsrc, GstAppStreamType type)
  * with gst_app_src_set_stream_type().
  *
  * Returns: the stream type.
- * 
+ *
  * Since: 0.10.22
  */
 GstAppStreamType
@@ -1253,7 +1243,7 @@ gst_app_src_get_stream_type (GstAppSrc * appsrc)
  * Set the maximum amount of bytes that can be queued in @appsrc.
  * After the maximum amount of bytes are queued, @appsrc will emit the
  * "enough-data" signal.
- * 
+ *
  * Since: 0.10.22
  */
 void
@@ -1282,7 +1272,7 @@ gst_app_src_set_max_bytes (GstAppSrc * appsrc, guint64 max)
  * Get the maximum amount of bytes that can be queued in @appsrc.
  *
  * Returns: The maximum amount of bytes that can be queued.
- * 
+ *
  * Since: 0.10.22
  */
 guint64
@@ -1336,7 +1326,7 @@ gst_app_src_set_latencies (GstAppSrc * appsrc, gboolean do_min, guint64 min,
  *
  * Configure the @min and @max latency in @src. If @min is set to -1, the
  * default latency calculations for pseudo-live sources will be used.
- * 
+ *
  * Since: 0.10.22
  */
 void
@@ -1352,7 +1342,7 @@ gst_app_src_set_latency (GstAppSrc * appsrc, guint64 min, guint64 max)
  * @max: the min latency
  *
  * Retrieve the min and max latencies in @min and @max respectively.
- * 
+ *
  * Since: 0.10.22
  */
 void
@@ -1403,7 +1393,7 @@ gst_app_src_set_emit_signals (GstAppSrc * appsrc, gboolean emit)
  *
  * Check if appsrc will emit the "new-preroll" and "new-buffer" signals.
  *
- * Returns: %TRUE if @appsrc is emiting the "new-preroll" and "new-buffer"
+ * Returns: %TRUE if @appsrc is emitting the "new-preroll" and "new-buffer"
  * signals.
  *
  * Since: 0.10.23
@@ -1528,7 +1518,7 @@ eos:
  * Returns: #GST_FLOW_OK when the buffer was successfuly queued.
  * #GST_FLOW_WRONG_STATE when @appsrc is not PAUSED or PLAYING.
  * #GST_FLOW_UNEXPECTED when EOS occured.
- * 
+ *
  * Since: 0.10.22
  */
 GstFlowReturn
@@ -1554,7 +1544,7 @@ gst_app_src_push_buffer_action (GstAppSrc * appsrc, GstBuffer * buffer)
  *
  * Returns: #GST_FLOW_OK when the EOS was successfuly queued.
  * #GST_FLOW_WRONG_STATE when @appsrc is not PAUSED or PLAYING.
- * 
+ *
  * Since: 0.10.22
  */
 GstFlowReturn
@@ -1567,7 +1557,7 @@ gst_app_src_end_of_stream (GstAppSrc * appsrc)
   priv = appsrc->priv;
 
   g_mutex_lock (priv->mutex);
-  /* can't accept buffers when we are flushing. We can accept them when we are 
+  /* can't accept buffers when we are flushing. We can accept them when we are
    * EOS although it will not do anything. */
   if (priv->flushing)
     goto flushing;
@@ -1600,7 +1590,7 @@ flushing:
  * This is an alternative to using the signals, it has lower overhead and is thus
  * less expensive, but also less flexible.
  *
- * If callbacks are installed, no signals will be emited for performance
+ * If callbacks are installed, no signals will be emitted for performance
  * reasons.
  *
  * Since: 0.10.23
